/**
 * Copyright 2010 Västra Götalandsregionen
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of version 2.1 of the GNU Lesser General Public
 *   License as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *   Boston, MA 02111-1307  USA
 */

package se.vgregion.ifeed.formbean;

import java.io.Serializable;
import java.util.AbstractList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import se.vgregion.ifeed.service.solr.DateFormatter;
import se.vgregion.ifeed.service.solr.DateFormatter.DateFormat;
import se.vgregion.ifeed.service.solr.DateUtils;

public class SearchResultList extends AbstractList<SearchResultList.SearchResult> implements Serializable {
    private static final long serialVersionUID = 1L;
    private List<Map<String, Object>> iFeedResults = null;
    private Iterator<Map<String, Object>> iterator = null;

    public SearchResultList(List<Map<String, Object>> iFeedResults) {
        this.iFeedResults = iFeedResults;
        iterator = iFeedResults.iterator();
    }

    public static class SearchResult {
        private Date processingTime;
        private String title;
        private String link;
        private String documentId;

        public SearchResult(Date processingTime, String title, String link, String documentId) {
            this.processingTime = processingTime;
            this.title = title;
            this.link = link;
            this.documentId = documentId;
        }

        public String getProcessingTime() {
            if(DateUtils.isToday(processingTime)) {
                return DateFormatter.format(processingTime, DateFormat.TIME_ONLY);
            } else {
                return DateFormatter.format(processingTime, DateFormat.DATE_ONLY);
            }
        }

        public String getTitle() {
            return title;
        }

        public String getLink() {
            return link;
        }

        public String getDocumentId() {
            return documentId;
        }
    }

    @Override
    public SearchResult get(int index) {
        final Map<String, Object> resultEntry = iFeedResults.get(index);
        final String documentId = (String)resultEntry.get("dc.identifier.documentid");
        final Date date = (Date)resultEntry.get("processingtime");
        final String title = (String)resultEntry.get("title");
        final String link = (String)resultEntry.get("url");
        return new SearchResult(new Date(date.getTime()), title, link, documentId);
    }

    @Override
    public int size() {
        return iFeedResults.size();
    }
}
